//-----------------------------------------------------------------------------
//
// Copyright (C) 2007, Xilinx, Inc. All Rights Reserved.
//
// This file is owned and controlled by Xilinx and must be used solely
// for design, simulation, implementation and creation of design files
// limited to Xilinx devices or technologies. Use with non-Xilinx
// devices or technologies is expressly prohibited and immediately
// terminates your license.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
//   ____  ____
//  /   /\/   /
// /___/  \  /    Vendor      : Xilinx
// \   \   \/     Version     : 1.0
//  \   \         Application : XAPP869
//  /   /         Filename    : frame_check.v
// /___/   /\     Module      : FRAME_CHECK
// \   \  /  \
//  \___\/\___\
//
//-----------------------------------------------------------------------------

//       This frame generator code is borrowed from Aurora IP and 
//       has been suitably modified to suit the requirement.
// 	   This logic checks frames received. 
//       Code is modified to support variable REM functionality.

`timescale 1 ns / 1 ps


module FRAME_CHECK
(
    // User Interface
    RX_D,  
    RX_REM,
    RX_SOF_N,
    RX_EOF_N,
    RX_SRC_RDY_N,  

    // System Interface
    USER_CLK,       
    RESET,
    ERROR_COUNT
  
);

//***********************************Port Declarations*******************************

   // User Interface
    input     [0:63]     RX_D;
    input     [0:2]      RX_REM;
    input                RX_SOF_N;
    input                RX_EOF_N;
    input                RX_SRC_RDY_N;
    
  // System Interface
    input                USER_CLK;
    input                RESET; 
    output    [0:7]      ERROR_COUNT;


//***************************Internal Register Declarations*************************** 

   reg                   in_frame_r; 
    reg        [63:0]       data_r;


   reg                   data_valid_r;
   reg                   error_detected_r;
   reg        [0:8]      error_count_r;
    
	reg  		  [0:2] 		rem_cnt;
   
//*********************************Wire Declarations**********************************

    wire                 in_frame_c;
    wire                 rem_valid_c;

    wire                 data_valid_c;
    wire                 error_detected_c;

//*********************************Main Body of Code**********************************


    //______________________________ Capture incoming data ___________________________    
    //Data is valid when RX_SRC_RDY_N is asserted

    assign  data_valid_c    =   in_frame_c && !RX_SRC_RDY_N && rem_valid_c;

    //Data is in a frame if it is a single cycle frame or a multi_cycle frame has started
    assign  in_frame_c  =   in_frame_r  ||  (!RX_SRC_RDY_N && !RX_SOF_N);


//Start a multicycle frame when a frame starts without ending on the same cycle.
		// End the frame when an EOF is detected
    always @(posedge USER_CLK)
        if(RESET)
            in_frame_r  <=      1'b0;
        else if(!in_frame_r && !RX_SOF_N && !RX_SRC_RDY_N && RX_EOF_N)
            in_frame_r  <=      1'b1;
        else if(in_frame_r && !RX_SRC_RDY_N && !RX_EOF_N)
            in_frame_r  <=      1'b0;

   	//__check for REM variation__//
	assign  rem_valid_c =   RX_EOF_N || (RX_REM == rem_cnt);
			//__counter to track REM__//
	always @(posedge USER_CLK)
		if (RESET)
			rem_cnt	<=	3'h7;
		else if (!RX_EOF_N)
			rem_cnt	<=	rem_cnt + 1'b1;


   //Capture valid incoming data, right shifted 1 bit for comparison with 
		//the next valid incoming data
		always @(posedge USER_CLK)
      	if (RESET)
         	data_r   <= {1'b1,63'd0};
         else if (data_valid_c)
            data_r   <= {data_r[0],data_r[63:1]};
    //Data in the data register is valid only if it was valid when captured and had no error
    always @(posedge USER_CLK)
        if(RESET)   data_valid_r    <=      1'b0;
        else        data_valid_r    <=      data_valid_c && !error_detected_c;

    
    //___________________________ Check incoming data for errors __________________________
         
    //An error is detected when valid data from the data register, 
				//when right shifted, does not match valid data
    								//from the RX port
	//__check data w.r.t. REM value__//
    assign  error_detected_c    =   in_frame_c && data_valid_c &&
					(RX_REM[0] ? (RX_D != data_r) : (RX_D[0:31] != data_r[63:32]));	
  
	//We register the error_detected signal for use with the error counter logic
    always @(posedge USER_CLK)
        if(RESET)  
            error_detected_r    <=      1'b0;
        else
            error_detected_r    <=      error_detected_c;  
    
    
    //We count the total number of errors we detect. By keeping a count we make it less likely that we will miss
    //errors we did not directly observe. This counter must be reset when it reaches its max value
    always @(posedge USER_CLK)
        if(RESET)
            error_count_r       <=      9'd0;
        else if(error_detected_r && !error_count_r[0] )
            error_count_r       <=      error_count_r + 1;
    
    //Here we connect the lower 8 bits of the count (the MSbit is used only to check when the counter reaches
    //max value) to the module output
    assign  ERROR_COUNT =   error_count_r[1:8];
    
    
endmodule           
