//-----------------------------------------------------------------------------
//
// Copyright (C) 2007, Xilinx, Inc. All Rights Reserved.
//
// This file is owned and controlled by Xilinx and must be used solely
// for design, simulation, implementation and creation of design files
// limited to Xilinx devices or technologies. Use with non-Xilinx
// devices or technologies is expressly prohibited and immediately
// terminates your license.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
// 
//   ____  ____
//  /   /\/   /
// /___/  \  /    Vendor      : Xilinx
// \   \   \/     Version     : 1.0
//  \   \         Application : XAPP869
//  /   /         Filename    : frame_gen.v
// /___/   /\     Module      : FRAME_GEN
// \   \  /  \
//  \___\/\___\
//
//-----------------------------------------------------------------------------

//			This frame generator code is borrowed from Aurora IP and 
//				has been suitably modified to suit the requirement.
// 		This logic generates frames of variable size. 
//			Code is modified to support variable REM functionality.


`timescale 1 ns / 1 ps

module FRAME_GEN
(
    // User Interface
    TX_D,  
    TX_REM,   
    TX_SOF_N,     
    TX_EOF_N,
    TX_SRC_RDY_N,
    TX_DST_RDY_N,

    // System Interface
    USER_CLK,       
    RESET
); 

//***********************************Port Declarations*******************************


   // User Interface
    output    [0:63]     TX_D;
    output    [0:2]      TX_REM;
    output               TX_SOF_N;
    output               TX_EOF_N;
  
    output               TX_SRC_RDY_N;
    input                TX_DST_RDY_N;
    
      // System Interface
    input                USER_CLK;
    input                RESET; 
	

//***************************External Register Declarations***************************

    reg                  TX_SRC_RDY_N;
    reg                  TX_SOF_N;
    reg                  TX_EOF_N;
	 reg  [0:2]				 TX_REM;

//***************************Internal Register Declarations*************************** 
    reg       [0:63]     tx_d_r;


    reg       [0:7]      frame_size_r;
    reg       [0:7]      bytes_sent_r;
    reg       [0:1]      ifg_size_r;

//State registers for onehot state machine
 
    reg                  idle_r;
    reg                  single_cycle_frame_r;
    reg                  sof_r;
    reg                  data_cycle_r;
    reg                  eof_r;

	 reg [2:0] 				 rem_cnt;

//*********************************Wire Declarations**********************************
    wire                 ifg_done_c;
   // Next state signals for one-hot state machine
    wire                 next_idle_c;
    wire                 next_single_cycle_frame_c;
    wire                 next_sof_c;
    wire                 next_data_cycle_c;
    wire                 next_eof_c;
//*********************************Main Body of Code**********************************

    //______________________________ Transmit Data  __________________________________    
    //Transmit data when TX_DST_RDY_N is asserted and not in an IFG. 
    
       always @(posedge USER_CLK)
        if(RESET)
        begin
            tx_d_r          <=      64'd1;

        end
        else if(!TX_DST_RDY_N && !idle_r)
        begin
            tx_d_r          <=      {tx_d_r[63],tx_d_r[0:62]};

        end
    
    //Connect TX_D to the internal tx_d_r register

    assign  TX_D    =   tx_d_r;
    


    //Use a counter to determine the size of the next frame to send
   always @(posedge USER_CLK)
        if(RESET)
            frame_size_r    <=      8'h00;
        else if(single_cycle_frame_r || eof_r)
            frame_size_r    <=      frame_size_r + 1;
 

    //Use a second counter to determine how many bytes of the frame have already been sent
    always @(posedge USER_CLK)
        if(RESET)
            bytes_sent_r    <=      8'h00;
        else if(sof_r)
            bytes_sent_r    <=      8'h01;
        else if(!TX_DST_RDY_N && !idle_r)
            bytes_sent_r    <=      bytes_sent_r + 1;


    //Use a freerunning counter to determine the IFG
    always @(posedge USER_CLK)
        if(RESET)
            ifg_size_r      <=      2'h0;
        else
            ifg_size_r      <=      ifg_size_r + 1;


    //IFG is done when ifg_size register is 0
    assign  ifg_done_c  =   (ifg_size_r == 2'h0);
		//--for back-to-back frame generation make ifg_done_c permanently high--//
    //assign  ifg_done_c  =   1'b1;

    //_____________________________ Framing State machine______________________________ 
    //Use a state machine to determine whether to start a frame, end a frame, send
    //data or send nothing
    
    //State registers for 1-hot state machine
    always @(posedge USER_CLK)
        if(RESET)
        begin
            idle_r                  <=      1'b1;
            single_cycle_frame_r    <=      1'b0;
            sof_r                   <=      1'b0;
            data_cycle_r            <=      1'b0;
            eof_r                   <=      1'b0;
        end
        else if(!TX_DST_RDY_N)
        begin
            idle_r                  <=      next_idle_c;
            single_cycle_frame_r    <=      next_single_cycle_frame_c;
            sof_r                   <=      next_sof_c;
            data_cycle_r            <=      next_data_cycle_c;
            eof_r                   <=      next_eof_c;
        end


    //Nextstate logic for 1-hot state machine
    assign  next_idle_c                 =   !ifg_done_c &&
                                            (single_cycle_frame_r || eof_r || idle_r);

    assign  next_single_cycle_frame_c   =   (ifg_done_c && (frame_size_r == 0)) &&
                                            (idle_r || single_cycle_frame_r || eof_r);

    assign  next_sof_c                  =   (ifg_done_c && (frame_size_r != 0)) &&
                                            (idle_r || single_cycle_frame_r || eof_r);

    assign  next_data_cycle_c           =   (frame_size_r != bytes_sent_r) &&
                                            (sof_r || data_cycle_r);

    assign  next_eof_c                  =   (frame_size_r == bytes_sent_r) &&
                                            (sof_r || data_cycle_r);


    //Output logic for 1-hot state machine
    always @(posedge USER_CLK)
        if(RESET)
        begin
            TX_SOF_N        <=      1'b1;
            TX_EOF_N        <=      1'b1;
            TX_SRC_RDY_N    <=      1'b1;
        end
        else if(!TX_DST_RDY_N)
        begin
            TX_SOF_N        <=      !(sof_r || single_cycle_frame_r);
            TX_EOF_N        <=      !(eof_r || single_cycle_frame_r);
            TX_SRC_RDY_N    <=      idle_r;
        end

			//__Counter to vary REM every frame__
		always @(posedge USER_CLK)
			if (RESET)
				rem_cnt	<=	3'h7;
			else if ((eof_r || single_cycle_frame_r) && !TX_DST_RDY_N)
				rem_cnt	<=	rem_cnt + 1'b1;

			//__REM variation with every frame__
		always @(posedge USER_CLK)
			if (RESET)
				TX_REM	<=	3'h7;
			else if (!TX_DST_RDY_N)
				TX_REM	<=	(eof_r || single_cycle_frame_r) ? rem_cnt : 3'h7;

endmodule 
